#!/usr/bin/python 
#
# shuber, 2007-07-07


import rubikcube;
import scene;

import sys;
import time;
from OpenGL.GL import *;
from OpenGL.GLUT import *;



class glrubikcube:

		#The scene 
		sc = None

		#The based rubikcube
		rc = None;
		#Size of cube
		size = 1.0
		#Gap between dices
		gap = 0.1

		#The slice which is highlighted
		highslice = 0
		#The rotation angle of the highlighted dice
		rot = 0


		#A command queue
		cmdQueue = []
		#The current index of the callback-function
		cmdFunc = None



		def __init__ (self, rc, sc):
				"""Create a OpenGL representation of a rubikcube"""

				#Set the rubikcube
				self.rc = rc;
				self.sc = sc;


		def keyboardFunc(self, key, x, y):
				"""Keyboard callback function"""

				#Highlight next slice
				if key == ' ':
					if glutGetModifiers() == 0:
						self.highlightNext()
					if glutGetModifiers() == GLUT_ACTIVE_ALT:
						self.highlightPrev()

				#Rotate slice
				if key == '\r':
					if glutGetModifiers() == 0:
						self.rotateSlice()
					if glutGetModifiers() == GLUT_ACTIVE_ALT:
						self.rotateSlice(True)



		def addCommand(self, cmd ):
				"""Add a new command to the command-queue"""

				#Add command to queue
				self.cmdQueue += [cmd]

				#If queue was empty before, add idle function
				if self.cmdFunc == None:
					self.cmdFunc = len(self.sc.idleFuncs)
					self.sc.addIdleFunc( lambda: self.idleProcCmd() )



		def idleProcCmd( self ):
				"""The processing function for the command queue. Will be executed
				when opengl calls idle function"""


				def execHighlightNext(self, cmd):

						self.highslice = (self.highslice+1) % (3*self.rc.size)
						return True

				def execHighlightPrev(self, cmd):
						
						self.highslice = (self.highslice-1) % (3*self.rc.size)
						return True

				def execHighlightSlice(self, cmd):

						self.highslice = cmd[1] % (3*self.rc.size)
						return True
			
				
				def execRotateSlice(self, cmd):

						#Unpacking parameters
						[c, reverse, delay, fps, start] = cmd

						#Set start time
						if start==None:
							start = time.time()
							cmd[4] = start
					
						if not reverse:
							deg = 90
						else:
							deg = -90
												
						#New rotation angle
						self.rot = deg * (time.time()-start)/delay

						#Reached target?
						if abs(self.rot) >= abs(deg):

							self.rot = 0
							self.rc.rotate( self.highslice, reverse)
							return True

						return False




				#If there is no command, quit idle function
				if len(self.cmdQueue) == 0:

					self.sc.removeIdleFunc(self.cmdFunc)
					self.cmdFunc = None
					return


				#This is the current command...
				currCmd = self.cmdQueue[0]


				#Dispatch command
				if currCmd[0] == 0:
						if not execHighlightNext(self, currCmd):
							return

				if currCmd[0] == 1:
						if not execHighlightPrev(self, currCmd):
							return

				if currCmd[0] == 2:
						if not execRotateSlice(self, currCmd):
							return

				if currCmd[0] == 3:
						if not execHighlightSlice(self, currCmd):
							return


				#Command finished, remove it from list
				del self.cmdQueue[0]




		def highlightSlice(self, idx):
				"""Highlight specific slice"""
				self.addCommand([3,idx])


		def highlightNext(self):
				"""Highlight next slice"""
				self.addCommand([0])


		def highlightPrev(self):
				"""Highlight previous slice"""
				self.addCommand([1])


		def rotateSlice(self, reverse=False, delay=0.25, fps=20.0):
				"""Rotate current slice"""
				self.addCommand([2, reverse, delay, fps, None])



		def glDisplay(self):

				def glSetMaterial( number, highlight ):

						if number==0:
							mat = [0.4, 0.4, 0.4, 0]

						elif number==1:
							mat = [1, 1, 1, 0]

						elif number==2:
							mat = [1, 0, 0, 0]

						elif number==3:
							mat = [0, 1, 0, 0]

						elif number==4:
							mat = [0, 0, 1, 0]

						elif number==5:
							mat = [0, 1, 1, 0]

						elif number==6:
							mat = [1, 0, 1, 0]

						else:
							assert(False);


						glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, mat );
						glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, map(lambda x: x*0.1, mat) );
						glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, map(lambda x: x*0.1, mat) );

						if highlight:
							glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, map(lambda x: x*0.8, mat) );
						else:
							glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, map(lambda x: x*0, mat) );



				def glDisplayDice( x, y, z, size, numbers, highlight  ):

						assert( len(numbers) == 6 )

						#Top face
						glBegin(GL_QUADS)
						glSetMaterial(numbers[0], highlight);
						glVertex3f( x, y+size, z )
						glVertex3f( x, y+size, z+size)
						glVertex3f( x+size, y+size, z+size)
						glVertex3f( x+size, y+size, z )
						glEnd()

						#Front face
						glBegin(GL_QUADS)
						glSetMaterial(numbers[1], highlight);
						glVertex3f( x, y, z+size )
						glVertex3f( x, y+size, z+size)
						glVertex3f( x+size, y+size, z+size)
						glVertex3f( x+size, y, z+size )
						glEnd()

						#Bottom face
						glBegin(GL_QUADS)
						glSetMaterial(numbers[2], highlight);
						glVertex3f( x, y, z )
						glVertex3f( x+size, y, z )
						glVertex3f( x+size, y, z+size)
						glVertex3f( x, y, z+size)
						glEnd()

						#Back face
						glBegin(GL_QUADS)
						glSetMaterial(numbers[3], highlight);
						glVertex3f( x, y, z )
						glVertex3f( x+size, y, z )
						glVertex3f( x+size, y+size, z)
						glVertex3f( x, y+size, z)
						glEnd()

						#Left face
						glBegin(GL_QUADS)
						glSetMaterial(numbers[4], highlight);
						glVertex3f( x, y, z )
						glVertex3f( x, y+size, z )
						glVertex3f( x, y+size, z+size)
						glVertex3f( x, y, z+size)
						glEnd()

						#Right face
						glBegin(GL_QUADS)
						glSetMaterial(numbers[5], highlight);
						glVertex3f( x+size, y, z )
						glVertex3f( x+size, y, z+size)
						glVertex3f( x+size, y+size, z+size)
						glVertex3f( x+size, y+size, z )
						glEnd()




				#Netto size of dices
				brutto = self.size/self.rc.size
				netto = brutto - self.gap
				max = self.rc.size

				for k in range(max):
					for j in range(max):
						for i in range(max):


							#The numbers for each face of the dice
							numbers = [0, 0, 0, 0, 0, 0]
							
							if j==max-1:
								numbers[0] = self.rc.pads[0][i][k]

							if k==max-1:
								numbers[1] = self.rc.pads[1][i][max-1-j]

							if j==0:
								numbers[2] = self.rc.pads[2][i][max-1-k]

							if k==0:
								numbers[3] = self.rc.pads[3][i][j]

							if i==0:
								numbers[4] = self.rc.pads[4][k][max-1-j]

							if i==max-1:
								numbers[5] = self.rc.pads[5][max-1-k][max-1-j]


							#Determine whether highlighted
							highdim = self.highslice / max
							highnr = self.highslice % max
							highlight = [i,j,k][highdim]==highnr

							glPushMatrix();

							if highlight:
								glRotate(self.rot, highdim==0, highdim==1, highdim==2)

							#Determine coordinates
							x, y, z =  map(lambda x: x*brutto-self.size/2+self.gap/2, [i,j,k]) 
							#Print the dice
							glDisplayDice( x,y,z, netto, numbers, highlight )

							glPopMatrix();
				





if __name__ == "__main__":

		s = scene.scene("Test glrubikcube");

		rc = rubikcube.rubikcube(3)
		glrc = glrubikcube(rc, s)

		s.objects += [ glrc ];
		s.keyboardObj += [ glrc ];


		glutMainLoop();


