#!/usr/bin/python 
#
# shuber, 2007-07-07


import sys;
import time;

from OpenGL.GL import *;
from OpenGL.GLU import *;
from OpenGL.GLUT import *;



class scene:
		"""A scene contains objects which provide a glDisplay method. Every re-display
		calls this methods"""

		#The objects to display, they must provide a glDisplay method
		objects = [];
		#The objects for which the keyboard callback-function is called
		keyboardObj = [];

		#Idle functions which will be called
		idleFuncs = []

		#Rotation angles of this scene
		rotx = 0
		roty = 0

		#Zoom in z-direction
		zoomz = -2.5;


		def __init__(self, title):
				"""Create a new scene which renders some objects"""

				#Init the glut system
				glutInit(sys.argv);

				#Create a window to display in
				glutInitWindowSize(400, 400);
				glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE);
				glutCreateWindow(title);

				#Enabeling depth-test and set polygon mode
				glEnable(GL_DEPTH_TEST);
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
				glShadeModel(GL_SMOOTH);


				#Set first light
				glLightfv(GL_LIGHT0, GL_DIFFUSE, [0.8, 0.2, 0.2, 1.0]);
				glLightfv(GL_LIGHT0, GL_POSITION, [3, 3, 3, 1.0]);
				glEnable(GL_LIGHT0);
				
				#Set second light
				glLightfv(GL_LIGHT1, GL_DIFFUSE, [0.2, 0.2, 0.8, 1.0]);
				glLightfv(GL_LIGHT1, GL_POSITION, [-3, -3, -3, 1.0]);
				glEnable(GL_LIGHT1);

				#Set an ambient light
				glLightModelfv(GL_LIGHT_MODEL_AMBIENT, [0.8, 0.8, 0.8, 1]);	
				glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
				glEnable(GL_LIGHTING);


				#Set callback-functions
				glutDisplayFunc(self.displayFunc);
				glutKeyboardFunc(self.keyboardFunc);
				glutSpecialFunc(self.specialFunc);
				glutReshapeFunc(self.reshapeFunc);



		def addIdleFunc( self, func ):
				"""Add an idle function"""

				self.idleFuncs += [func]
				glutIdleFunc(self.idleFunc)

		def removeIdleFunc(self, idx ):
				"""Remove an idle function"""

				del self.idleFuncs[idx]

				if len(self.idleFuncs) == 0:
					glutIdleFunc(None)




		def idleFunc(self):
				"""The idle function"""

				for func in self.idleFuncs:
					func()

				glutPostRedisplay()


		def displayFunc(self):
				"""Display callback function"""

				glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
				glLoadIdentity();


				glTranslatef(0, 0, self.zoomz);

				#Rotate scene
				glRotatef( self.roty, 0, 1, 0  )
				glRotatef( self.rotx, 1, 0, 0  )

				#Display all scene objectes
				for obj in self.objects:
					obj.glDisplay();

				glutSwapBuffers();


		def keyboardFunc(self, key, x, y):
				"""Keyboard callback function"""
				
				#Pressed esc-key
				if key == "\x1b" :
					sys.exit(1);


				#Call the function
				for obj in self.keyboardObj:
					obj.keyboardFunc(key, x, y)


		def specialFunc(self, key, x, y):
				"""Special callback function"""
			

				#Pressed up-key
				if key == 101:
						self.rotx -= 5
						glutPostRedisplay()

				#Pressed down-key
				elif key == 103:
						self.rotx += 5
						glutPostRedisplay()

				#Pressed right key
				elif key == 102:
						self.roty += 5
						glutPostRedisplay()

				#Pressed left key
				elif key == 100:
						self.roty -= 5
						glutPostRedisplay()

				#Pressed page-up
				elif key == 104:
						self.zoomz += .25
						glutPostRedisplay()

				#Pressed page-down
				elif key == 105:
						self.zoomz -= .25
						glutPostRedisplay()

				else:
					print "Pressed unknown special key:", repr(key);


		def reshapeFunc(self, width, height):
				"""Reshape callback function"""

				#Reset viewport to new dimensions
				glViewport(0, 0, width, height);

				#Set the projection matrix
				glMatrixMode(GL_PROJECTION);
				glLoadIdentity();
				gluPerspective(60, 1, 1, 200);

				#Switch back to model-view matrix
				glMatrixMode(GL_MODELVIEW);



if __name__ == "__main__":

		s = scene("Scene test");
		glutMainLoop();


